@@ -681,6 +681,7 @@ where
     fn bubble_up(&mut self, mut position: usize, map_position: usize) -> usize {
         if position > 0 {
             position = if level(position) % 2 == 0 {
+                //on a min level
                 if self
                     .store
                     .map
@@ -689,6 +690,7 @@ where
                     .1
                     < self.store.map.get_index(map_position).unwrap().1
                 {
+                    // and greater then parent
                     unsafe {
                         *self.store.heap.get_unchecked_mut(position) =
                             *self.store.heap.get_unchecked(parent(position));
@@ -699,6 +701,7 @@ where
                     }
                     self.bubble_up_max(parent(position), map_position)
                 } else {
+                    // and less then parent
                     self.bubble_up_min(position, map_position)
                 }
             } else if self
@@ -709,6 +712,7 @@ where
                 .1
                 > self.store.map.get_index(map_position).unwrap().1
             {
+                // on a max level and less then parent
                 unsafe {
                     *self.store.heap.get_unchecked_mut(position) =
                         *self.store.heap.get_unchecked(parent(position));
@@ -719,6 +723,7 @@ where
                 }
                 self.bubble_up_min(parent(position), map_position)
             } else {
+                // on a max level and greater then parent
                 self.bubble_up_max(position, map_position)
             }
         }
@@ -742,7 +747,14 @@ where
                 .1
                 > self.store.map.get_index(map_position).unwrap().1)
         {
-            self.store.swap(position, parent(parent(position)));
+            unsafe {
+                *self.store.heap.get_unchecked_mut(position) =
+                    *self.store.heap.get_unchecked(parent(parent(position)));
+                *self
+                    .store
+                    .qp
+                    .get_unchecked_mut(*self.store.heap.get_unchecked(position)) = position;
+            }
             position = parent(parent(position));
         }
         position
@@ -758,7 +770,14 @@ where
                 .1
                 < self.store.map.get_index(map_position).unwrap().1)
         {
-            self.store.swap(position, parent(parent(position)));
+            unsafe {
+                *self.store.heap.get_unchecked_mut(position) =
+                    *self.store.heap.get_unchecked(parent(parent(position)));
+                *self
+                    .store
+                    .qp
+                    .get_unchecked_mut(*self.store.heap.get_unchecked(position)) = position;
+            }
             position = parent(parent(position));
         }
         position
